//To compile the project you need the extended version of the CADImport .NET library
//that allows to access CAD entities directly.  You can get this version of the 
//library free of charge by emailing your request to: info@cadsofttools.com
using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Globalization;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using CADImport;
using CADImport.CADImportForms;
using CADImport.FaceModule;
using CADImport.RasterImage;
using CADImport.HPGL2;
using CADImport.Export.DirectCADtoDXF;

namespace CADImportNetDemos.AddEntityDemo
{
	#region Help
	/// <summary>
	/// Represents the main application form where CAD image and added CAD entities can be viewed.
	/// </summary>
	#endregion Help
	public class MainForm: System.Windows.Forms.Form
	{	
		private CADImport.CADImportForms.LayerForm lForm;
		private CADImport.CADImportForms.AboutForm aboutFrm;		
		private CADImport.CADImportForms.SetRasterSizeForm rasterSizeForm;
		private SHXForm frmSHX;
		private AddCircleForm createCircleForm;
		private AddLineForm createLineForm;
		private AddPolyForm createPolyForm;
		private AddTextForm createTextForm;
		private AddMTextForm createMTextForm;		
		#region protect
#if protect
		private RegForm regFrm;
#endif
#if (protect && floatprotect)
			private FloatLicForm floatLicFrm;		
#endif
		#endregion protect				

		private System.Drawing.PointF old_Pos;	
		private System.Drawing.PointF pos;
		private System.Drawing.SizeF visibleArea;
		private float sc;
		private float prev_sc;	
		private int cX;		
		private int cY;		
		private bool detRMouseDown;								
		private bool drawMode;		
		private bool useSHXFonts;	
		private int numlastadd;			
		private CADImport.CADImage cadImage;		
		private CADImport.FaceModule.ClipRect clipRectangle;									
		private CADImport.FaceModule.ListItemSelectEventHandler listItemSelectDel;
		private static string openFileName;					
		private CADImport.FaceModule.CADPictureBox cadPictBox;		
		
		private System.Windows.Forms.StatusBar stBar;
		private System.Windows.Forms.StatusBarPanel sbOpen;
		private System.Windows.Forms.StatusBarPanel sbScale;
		private System.Windows.Forms.ToolBarButton tlbOpen;
		private System.Windows.Forms.ToolBarButton tlbZoomIn;
		private System.Windows.Forms.ToolBarButton tlbZoomOut;
		private System.Windows.Forms.ToolBarButton tlbLay;
		private System.Windows.Forms.ToolBarButton tlbWhite;
		private System.Windows.Forms.ToolBarButton tlbBlack;
		private System.Windows.Forms.ToolBarButton tlbAddEnt;
		private System.Windows.Forms.ToolBarButton tlbSave;
		private System.Windows.Forms.ToolBar tlbTool;
		private System.Windows.Forms.StatusBarPanel sbCoord;
		private System.Windows.Forms.ImageList toolBtnImageList;
		private System.ComponentModel.IContainer components;		
		private ContextMenu addContextMenu;
		private MenuItem addLineMenuItem;
		private MenuItem addPolyLineMenuItem;
		private MenuItem addCircleMenuItem;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItem5;
		private System.Windows.Forms.MenuItem menuItem7;		
		private System.Windows.Forms.MenuItem menuItem8;		
		private System.Windows.Forms.ToolBarButton tlbShx;
		private System.Windows.Forms.MenuItem menuItem9;		
		private System.Windows.Forms.MenuItem miFloatLic;
		private System.Windows.Forms.OpenFileDialog dlgOpenFile;
		private System.Windows.Forms.MenuItem miOpenFile;
		private System.Windows.Forms.MenuItem miSaveAsDXF;
		private System.Windows.Forms.MenuItem miSep;
		private System.Windows.Forms.MenuItem miExit;
		private System.Windows.Forms.MenuItem miSHXForm;
		private System.Windows.Forms.MenuItem miRegister;
		private System.Windows.Forms.MenuItem miAbout;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.SaveFileDialog dlgSaveImg;
		private System.Windows.Forms.SaveFileDialog dlgSaveDXF;		
		private System.Windows.Forms.ColorDialog setColorDlg;				
		
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.dlgOpenFile = new System.Windows.Forms.OpenFileDialog();
			this.stBar = new System.Windows.Forms.StatusBar();
			this.sbOpen = new System.Windows.Forms.StatusBarPanel();
			this.sbScale = new System.Windows.Forms.StatusBarPanel();
			this.sbCoord = new System.Windows.Forms.StatusBarPanel();
			this.tlbTool = new System.Windows.Forms.ToolBar();
			this.tlbOpen = new System.Windows.Forms.ToolBarButton();
			this.tlbZoomIn = new System.Windows.Forms.ToolBarButton();
			this.tlbZoomOut = new System.Windows.Forms.ToolBarButton();
			this.tlbWhite = new System.Windows.Forms.ToolBarButton();
			this.tlbBlack = new System.Windows.Forms.ToolBarButton();
			this.tlbLay = new System.Windows.Forms.ToolBarButton();
			this.tlbAddEnt = new System.Windows.Forms.ToolBarButton();
			this.addContextMenu = new System.Windows.Forms.ContextMenu();
			this.addCircleMenuItem = new System.Windows.Forms.MenuItem();
			this.addLineMenuItem = new System.Windows.Forms.MenuItem();
			this.addPolyLineMenuItem = new System.Windows.Forms.MenuItem();
			this.menuItem7 = new System.Windows.Forms.MenuItem();
			this.menuItem8 = new System.Windows.Forms.MenuItem();
			this.tlbSave = new System.Windows.Forms.ToolBarButton();
			this.tlbShx = new System.Windows.Forms.ToolBarButton();
			this.toolBtnImageList = new System.Windows.Forms.ImageList(this.components);
			this.dlgSaveImg = new System.Windows.Forms.SaveFileDialog();
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.miOpenFile = new System.Windows.Forms.MenuItem();
			this.miSaveAsDXF = new System.Windows.Forms.MenuItem();
			this.miSep = new System.Windows.Forms.MenuItem();
			this.miExit = new System.Windows.Forms.MenuItem();
			this.menuItem9 = new System.Windows.Forms.MenuItem();
			this.miSHXForm = new System.Windows.Forms.MenuItem();
			this.menuItem5 = new System.Windows.Forms.MenuItem();
			this.miRegister = new System.Windows.Forms.MenuItem();
			this.miFloatLic = new System.Windows.Forms.MenuItem();
			this.miAbout = new System.Windows.Forms.MenuItem();
			this.dlgSaveDXF = new System.Windows.Forms.SaveFileDialog();
			this.setColorDlg = new System.Windows.Forms.ColorDialog();
			this.cadPictBox = new CADImport.FaceModule.CADPictureBox();
			((System.ComponentModel.ISupportInitialize)(this.sbOpen)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbScale)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbCoord)).BeginInit();
			this.SuspendLayout();
			// 
			// dlgOpenFile
			// 
			this.dlgOpenFile.DefaultExt = "*.dxf;*.dwg";
			this.dlgOpenFile.Filter = @"All(*.dxf,*.dwg,*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl,*.cgm, *.bmp,*.jpg, *.jpeg,*.tiff, *.gif, *.ico, *.cur, *.png, *.emf, *.wmf)|*.bmp;*.jpg;*.jpeg;*.tiff; *.gif; *.ico;*.dxf;*.dwg;*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl;*.cur;*.png; *.emf; *.wmf; *.cgm|CAD files(*.dxf, *.dwg)|*.dxf;*.dwg|All raster files (*.bmp,*.jpg, *.jpeg,*.tiff,*.tif, *.gif, *.ico, *.cur, *.png)|*.bmp;*.jpg;*.jpeg;*.tiff;*.tif; *.gif;*.ico;*.cur; *.png|Metafiles ( *.emf, *.wmf)| *.emf; *.wmf|DXF files(*.dxf)|*.dxf|DWG files(*.dwg)|*.dwg|BMP(*.bmp)|*.bmp|JPG(*.jpg, *.jpeg)| *.jpg; *.jpeg|TIFF(*.tiff, *.tif)|*.tiff;*.tif|GIF(*.gif)|*.gif|ICO(*.ico)|*.ico|PNG(*.png)|*.png|Cursors(*.cur)|*.cur|HPGL(*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl)|*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl|CGM (*.cgm)|*.cgm";
			this.dlgOpenFile.ShowHelp = true;
			// 
			// stBar
			// 
			this.stBar.Font = new System.Drawing.Font("Microsoft Sans Serif", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.stBar.Location = new System.Drawing.Point(0, 390);
			this.stBar.Name = "stBar";
			this.stBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																					 this.sbOpen,
																					 this.sbScale,
																					 this.sbCoord});
			this.stBar.ShowPanels = true;
			this.stBar.Size = new System.Drawing.Size(687, 16);
			this.stBar.TabIndex = 9;
			// 
			// sbOpen
			// 
			this.sbOpen.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbOpen.MinWidth = 100;
			this.sbOpen.ToolTipText = "File Name";
			// 
			// sbScale
			// 
			this.sbScale.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.sbScale.ToolTipText = "Scale";
			this.sbScale.Width = 520;
			// 
			// sbCoord
			// 
			this.sbCoord.Alignment = System.Windows.Forms.HorizontalAlignment.Right;
			this.sbCoord.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbCoord.Text = "0 - 0 - 0";
			this.sbCoord.Width = 51;
			// 
			// tlbTool
			// 
			this.tlbTool.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.tlbTool.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.tlbTool.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																					   this.tlbOpen,
																					   this.tlbZoomIn,
																					   this.tlbZoomOut,
																					   this.tlbWhite,
																					   this.tlbBlack,
																					   this.tlbLay,
																					   this.tlbAddEnt,
																					   this.tlbSave,
																					   this.tlbShx});
			this.tlbTool.ButtonSize = new System.Drawing.Size(20, 20);
			this.tlbTool.DropDownArrows = true;
			this.tlbTool.ImageList = this.toolBtnImageList;
			this.tlbTool.Location = new System.Drawing.Point(0, 0);
			this.tlbTool.Name = "tlbTool";
			this.tlbTool.ShowToolTips = true;
			this.tlbTool.Size = new System.Drawing.Size(687, 29);
			this.tlbTool.TabIndex = 10;
			this.tlbTool.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.tlbTool_ButtonClick);
			// 
			// tlbOpen
			// 
			this.tlbOpen.ImageIndex = 1;
			this.tlbOpen.ToolTipText = "Open file";
			// 
			// tlbZoomIn
			// 
			this.tlbZoomIn.Enabled = false;
			this.tlbZoomIn.ImageIndex = 2;
			this.tlbZoomIn.ToolTipText = "Zoom In(+)";
			// 
			// tlbZoomOut
			// 
			this.tlbZoomOut.Enabled = false;
			this.tlbZoomOut.ImageIndex = 3;
			this.tlbZoomOut.ToolTipText = "Zoom Out(-)";
			// 
			// tlbWhite
			// 
			this.tlbWhite.Enabled = false;
			this.tlbWhite.ImageIndex = 4;
			this.tlbWhite.ToolTipText = "White background";
			// 
			// tlbBlack
			// 
			this.tlbBlack.Enabled = false;
			this.tlbBlack.ImageIndex = 5;
			this.tlbBlack.ToolTipText = "Black background";
			// 
			// tlbLay
			// 
			this.tlbLay.Enabled = false;
			this.tlbLay.ImageIndex = 6;
			this.tlbLay.ToolTipText = "Show layers form";
			// 
			// tlbAddEnt
			// 
			this.tlbAddEnt.DropDownMenu = this.addContextMenu;
			this.tlbAddEnt.Enabled = false;
			this.tlbAddEnt.ImageIndex = 0;
			this.tlbAddEnt.Style = System.Windows.Forms.ToolBarButtonStyle.DropDownButton;
			this.tlbAddEnt.ToolTipText = "Add entity";
			// 
			// addContextMenu
			// 
			this.addContextMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						   this.addCircleMenuItem,
																						   this.addLineMenuItem,
																						   this.addPolyLineMenuItem,
																						   this.menuItem7,
																						   this.menuItem8});
			// 
			// addCircleMenuItem
			// 
			this.addCircleMenuItem.Index = 0;
			this.addCircleMenuItem.Text = "Circle";
			this.addCircleMenuItem.Click += new System.EventHandler(this.addEntity_Click);
			// 
			// addLineMenuItem
			// 
			this.addLineMenuItem.Index = 1;
			this.addLineMenuItem.Text = "Line";
			this.addLineMenuItem.Click += new System.EventHandler(this.addEntity_Click);
			// 
			// addPolyLineMenuItem
			// 
			this.addPolyLineMenuItem.Index = 2;
			this.addPolyLineMenuItem.Text = "Polyline";
			this.addPolyLineMenuItem.Click += new System.EventHandler(this.addEntity_Click);
			// 
			// menuItem7
			// 
			this.menuItem7.Index = 3;
			this.menuItem7.Text = "Text";
			this.menuItem7.Click += new System.EventHandler(this.addEntity_Click);
			// 
			// menuItem8
			// 
			this.menuItem8.Index = 4;
			this.menuItem8.Text = "MText";
			this.menuItem8.Click += new System.EventHandler(this.addEntity_Click);
			// 
			// tlbSave
			// 
			this.tlbSave.Enabled = false;
			this.tlbSave.ImageIndex = 7;
			this.tlbSave.ToolTipText = "Save image";
			// 
			// tlbShx
			// 
			this.tlbShx.Enabled = false;
			this.tlbShx.ImageIndex = 8;
			this.tlbShx.Pushed = true;
			// 
			// toolBtnImageList
			// 
			this.toolBtnImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.toolBtnImageList.ImageSize = new System.Drawing.Size(16, 16);
			this.toolBtnImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("toolBtnImageList.ImageStream")));
			this.toolBtnImageList.TransparentColor = System.Drawing.Color.White;
			// 
			// dlgSaveImg
			// 
			this.dlgSaveImg.Filter = "*.bmp|*.bmp|*.jpg|*.jpg|*.tiff|*.tiff|*.gif|*.gif|*.emf|*.emf|*.dxf|*.dxf";
			this.dlgSaveImg.RestoreDirectory = true;
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.menuItem1,
																					 this.menuItem9,
																					 this.menuItem5});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.miOpenFile,
																					  this.miSaveAsDXF,
																					  this.miSep,
																					  this.miExit});
			this.menuItem1.Text = "File";
			// 
			// miOpenFile
			// 
			this.miOpenFile.Index = 0;
			this.miOpenFile.Text = "Open File ...";
			this.miOpenFile.Click += new System.EventHandler(this.miOpenFile_Click);
			// 
			// miSaveAsDXF
			// 
			this.miSaveAsDXF.Enabled = false;
			this.miSaveAsDXF.Index = 1;
			this.miSaveAsDXF.Text = "Save as DXF...";
			this.miSaveAsDXF.Click += new System.EventHandler(this.miSaveAsDXF_Click);
			// 
			// miSep
			// 
			this.miSep.Index = 2;
			this.miSep.Text = "-";
			// 
			// miExit
			// 
			this.miExit.Index = 3;
			this.miExit.Text = "Close";
			this.miExit.Click += new System.EventHandler(this.miExit_Click);
			// 
			// menuItem9
			// 
			this.menuItem9.Index = 1;
			this.menuItem9.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.miSHXForm});
			this.menuItem9.Text = "View";
			// 
			// miSHXForm
			// 
			this.miSHXForm.Index = 0;
			this.miSHXForm.Text = "SHX Fonts";
			this.miSHXForm.Click += new System.EventHandler(this.miSHXForm_Click);
			// 
			// menuItem5
			// 
			this.menuItem5.Index = 2;
			this.menuItem5.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.miRegister,
																					  this.miFloatLic,
																					  this.miAbout});
			this.menuItem5.Text = "?";
			// 
			// miRegister
			// 
			this.miRegister.Index = 0;
			this.miRegister.Text = "Register";
			this.miRegister.Click += new System.EventHandler(this.miRegister_Click);
			// 
			// miFloatLic
			// 
			this.miFloatLic.Index = 1;
			this.miFloatLic.Text = "Floating License Registration...";
			this.miFloatLic.Click += new System.EventHandler(this.miFloatLic_Click);
			// 
			// miAbout
			// 
			this.miAbout.Index = 2;
			this.miAbout.Text = "About";
			this.miAbout.Click += new System.EventHandler(this.miAbout_Click);
			// 
			// dlgSaveDXF
			// 
			this.dlgSaveDXF.DefaultExt = "*.dxf";
			this.dlgSaveDXF.Filter = "*.dxf|*.dxf";
			// 
			// cadPictBox
			// 
			this.cadPictBox.BackColor = System.Drawing.Color.Black;
			this.cadPictBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.cadPictBox.DoubleBuffering = true;
			this.cadPictBox.Location = new System.Drawing.Point(0, 29);
			this.cadPictBox.Name = "cadPictBox";
			this.cadPictBox.Position = new System.Drawing.Point(0, 0);
			this.cadPictBox.Size = new System.Drawing.Size(687, 361);
			this.cadPictBox.TabIndex = 11;
			this.cadPictBox.TabStop = false;
			this.cadPictBox.VirtualSize = new System.Drawing.Size(0, 0);
			this.cadPictBox.MouseUp += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseUp);
			this.cadPictBox.Paint += new System.Windows.Forms.PaintEventHandler(this.cadPictBox_Paint);
			this.cadPictBox.MouseMove += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseMove);
			this.cadPictBox.MouseDown += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseDown);
			// 
			// AddEntityForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.SystemColors.Control;
			this.ClientSize = new System.Drawing.Size(687, 406);
			this.Controls.Add(this.cadPictBox);
			this.Controls.Add(this.tlbTool);
			this.Controls.Add(this.stBar);
			this.Menu = this.mainMenu;
			this.Name = "AddEntityForm";
			this.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "CADImport.NET - Add Entity";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.Resize += new System.EventHandler(this.MainForm_Resize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
			this.Load += new System.EventHandler(this.MainForm_Load);
			this.Deactivate += new System.EventHandler(this.MainForm_Deactivate);
			((System.ComponentModel.ISupportInitialize)(this.sbOpen)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbScale)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbCoord)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion		

		#region Help
#if	RUSHELP
		/// <summary>
		///       - 
		/// (    true - )
		/// </summary>
#else
		/// <summary>
		/// Gets or sets a flag of the drawing mode
		/// (if <b>true</b> - in color, else - in black-and-white)
		/// </summary>
		#endif
		#endregion
		public bool DrawMode
		{
			get
			{
				return this.drawMode;
			}
			set
			{				
				this.drawMode = value;
			}
		}		

		#region Help
		/// <summary>
		/// A CAD image displayed in the main application form.
		/// </summary>
		#endregion Help		
		public CADImage Image
		{
			get
			{
				return cadImage;
			}
			set
			{
				this.cadImage = value;
			}
		}

		#region Help
		/// <summary>
		/// An X coordinate of the last mouse click position on the screen.
		/// </summary>
		#endregion Help
		public int CurrentXClickPosition
		{
			get
			{
				return this.cX;
			}
		}
		#region Help
		/// <summary>
		/// A Y coordinate of the last mouse click position on the screen.
		/// </summary>
		#endregion Help
		public int CurrentYClickPosition
		{
			get
			{
				return this.cY;
			}
		}		

		#region Help
		/// <summary>
		/// A previous position of the current CAD image in relation to the top-left corner of the viewer.
		/// </summary>
		#endregion Help
		public PointF PreviousPosition
		{
			get
			{
				return this.old_Pos;
			}
			set
			{
				this.old_Pos = value;
			}
		}			
	
		#region Help
#if RUSHELP
		/// <summary>
		///      
		/// </summary>
#else		
		/// <summary>
		/// A position of the current CAD image in relation to the top-left corner of the viewer.
		/// </summary>		
#endif
		#endregion
		public PointF Position
		{
			get
			{
				return this.pos;
			}
			set
			{
				this.pos = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///         X
		/// </summary>
#else		
		/// <summary>
		/// Gets or sets a position of the drawing image by X axis
		/// </summary>
	
#endif
		#endregion
		public float LeftImagePosition
		{
			get
			{
				return this.pos.X;
			}
			set
			{
				this.pos.X = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///         Y
		/// </summary>
#else
		/// <summary>
		/// Gets or sets a position of the drawing image by Y axis
		/// </summary>
#endif
		#endregion
		public float TopImagePosition
		{
			get
			{
				return this.pos.Y;
			}
			set
			{
				this.pos.Y = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///        
		/// </summary>
#else
		/// <summary>
		/// Gets or sets base dimensions of the image drawing area
		/// </summary>
#endif
		#endregion
		public SizeF VisibleAreaSize
		{
			get
			{
				return this.visibleArea;
			}
			set
			{
				this.visibleArea = value;
			}
		}

		#region Help
		/// <summary>
		/// A scale in which the current CAD image is displayed in the viewer.
		/// </summary>
		#endregion Help
		public float ImageScale
		{
			get
			{
				return this.sc;
			}
			set
			{
				this.sc = value;
			}
		}

		#region Help
		/// <summary>
		/// A previous value of the scale in which the current CAD image is displayed in the viewer.
		/// </summary>
		#endregion Help
		public float ImagePreviousScale
		{
			get
			{
				return this.prev_sc;
			}
			set
			{
				this.prev_sc = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///       
		/// </summary>
#else
		/// <summary>
		/// Gets the current size and position of the drawing image
		/// </summary>
#endif
		#endregion
		public RectangleF ImageRectangleF
		{
			get
			{
				return new RectangleF(this.LeftImagePosition, this.TopImagePosition, 
									  VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);								
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///       
		/// </summary>
#else
		/// <summary>
		/// Gets the current size and position of the drawing image
		/// </summary>
#endif
		#endregion
		public DRect ImageDRect
		{
			get
			{
				return new DRect(this.LeftImagePosition, this.TopImagePosition, 
								 VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);								
			}
		}

		private string RealScale
		{
			get
			{
				if(cadImage != null)
					return string.Format("{0,2:F}%", (((this.VisibleAreaSize.Width * ImageScale) / this.cadImage.AbsWidth) * cadImage.MMToPixelX) * 100.0);
				else 
					return string.Format("{0}%", ImageScale);
			}
		}

		[STAThread]
		static void Main(string[] args) 
		{						
			Application.EnableVisualStyles();
			Application.DoEvents();						
			LoadParams(args);
			Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(DoExceptionWorking);
			Application.Run(new MainForm());	
		}	

		private static void DoExceptionWorking(object sender, System.Threading.ThreadExceptionEventArgs args)
		{				
			CADConst.SendExceptionReport(args.Exception);
		}
	
		private static void LoadParams(string[] args)
		{
			if(args.Length != 0)	
				openFileName = args[0];							
		}

		#region Help
		/// <summary>
		/// Adjusts the image size according to the specified scale.
		/// </summary>
		/// <param name="i">A scale value.</param>
		/// <remarks>If the scale value is more than one the image size increases; otherwise, decreases.</remarks>
		#endregion Help
		public void Zoom(float i)
		{
			if(cadImage == null) 
				return;
			sc = sc * i;
			if(sc < 0.005f) 
				sc = 0.005f;	
			cadPictBox.Invalidate();
			cadPictBox_MouseMove(cadPictBox, new MouseEventArgs(MouseButtons.Left, 0, (int)old_Pos.X, (int)old_Pos.Y, 0));			
		}

		private void Shift()
		{
			pos.X = old_Pos.X - (old_Pos.X - pos.X) * sc / prev_sc;
			pos.Y = old_Pos.Y - (old_Pos.Y - pos.Y) * sc / prev_sc;
			prev_sc = sc;		
		}

		private void CADPictBoxMouseWheel(object sender, System.Windows.Forms.MouseEventArgs e)
		{			
			if(e.Delta < 0)	
				Zoom(0.7f);
			else 
				Zoom(1.3f);
			this.Shift();
			SetPictureBoxPosition(Position);
		}

		#region Help
		/// <summary>
		/// Initializes a new instance of the <see cref="MainForm">CADImportNetDemos.AddEntityDemo.MainForm</see> class.
		/// </summary>
		#endregion Help
		public MainForm()
		{			
			InitParams();
			InitializeComponent();			
			InitExtParam();
		}

		private void InitExtParam()
		{
			#region protect
			#if ((! floatprotect) && protect)
				this.miFloatLic.Enabled = false;
			#endif
			#endregion
			this.clipRectangle = new ClipRect(this.cadPictBox);
			this.cadPictBox.MouseWheel += new MouseEventHandler(CADPictBoxMouseWheel);
			this.cadPictBox.ScrollEvent += new CADImport.FaceModule.ScrollEventHandlerExt(CADPictBoxScroll);						
			this.createCircleForm.ResizeImageEvent += new CADImport.FaceModule.CADImportEventHandler(this.ResizeLayout);
		}

		private void InitParams()
		{
			sc = 1.0f;		
			useSHXFonts = true;
			numlastadd = 0;
			listItemSelectDel = new ListItemSelectEventHandler(ListSelect);					
			old_Pos = PointF.Empty;		
			lForm = new LayerForm();
			aboutFrm = new AboutForm();
			rasterSizeForm = new SetRasterSizeForm();
			createCircleForm = new AddCircleForm();
			createLineForm = new AddLineForm();
			createPolyForm = new AddPolyForm();
			createTextForm = new AddTextForm();
			createMTextForm = new AddMTextForm();			
			frmSHX = new SHXForm();
			#region protect
#if (protect && floatprotect)
				floatLicFrm = new FloatLicForm();
				floatLicFrm.Change += new System.EventHandler(InvalidateImage);
#endif
#if protect
			regFrm = new RegForm();
#endif
			#endregion protect
		}

		private void CADPictBoxScroll(object sender, CADImport.FaceModule.ScrollEventArgsExt e)
		{			
			LeftImagePosition = -this.cadPictBox.Position.X;
			TopImagePosition = -this.cadPictBox.Position.Y;
			this.cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Cleans up any resources being used.
		/// </summary>
		/// <param name="disposing">A value indicating if both managed and unmanaged resources have to be released (<b>true</b>) or only unmanaged (<b>false</b>). 
		///</param>
		#endregion Help
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		
		private void DeactCADImage()
		{			
			lForm.LayerList.Clear();
			lForm.LayerList.SelectItemEvent -= listItemSelectDel;			
		}

		private void ResizeLayout()
		{
			if(this.cadImage == null) 
				return;
			this.cadImage.GetExtents();
			this.DoResize(); 
		}

		#region Help
#if RUSHELP
		/// <summary>
		///        
		/// </summary>
#else
	/// <summary>
	/// Invokes open file dialog and loads specified file
	/// </summary>
#endif
		#endregion
		public void ShowLoadFileDialog()
		{
			DeactCADImage();
			this.cadPictBox.Enabled = false;
			if(dlgOpenFile.ShowDialog() != DialogResult.OK) 
			{
				cadPictBox.Visible = true;	
				this.cadPictBox.Enabled = true;
				return;
			}
			this.cadPictBox.Enabled = true;
			LoadFile(dlgOpenFile.FileName);			
		}

		private void Open_Click(object sender, System.EventArgs e)
		{
			this.ShowLoadFileDialog();	
			this.ChangeControlState();
		}

		private void InvalidateImage(object sender, System.EventArgs e)
		{
			this.cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Loads a CAD file.
		/// </summary>
		#endregion Help
		public void LoadFile(string fileName)
		{									
			if(fileName != null)
			{
				if(cadImage != null)
				{	
					cadImage.Dispose();
					cadImage = null;
					this.lForm.LayerList.Clear();
					lForm.LayerList.SelectItemEvent -= listItemSelectDel;			
				}
				this.Cursor = Cursors.WaitCursor;
				stBar.Panels[0].Text = ApplicationConstants.loadfilestr;
				ImageScale = 1.0f;
				ImagePreviousScale = 1.0f;
				Position = new PointF();				
				this.cadImage = CADImage.CreateImageByExtension(fileName);																
				if(this.cadImage is CADRasterImage)
					(this.cadImage as CADRasterImage).Control = this.cadPictBox;
			} 
			if(this.cadImage != null)
			{									
				if(CADConst.IsWebPath(fileName))
					this.cadImage.LoadFromWeb(fileName);
				else
					cadImage.LoadFromFile(fileName);								
			}
			stBar.Panels[0].Text = Path.GetFileName(fileName) + ApplicationConstants.sepstr;			
			CADImage.LastLoadedFilePath = Path.GetDirectoryName(fileName); 
			SetCADImageOptions();
		}

		#region Help
#if RUSHELP
		/// <summary>
		///     <see cref="MainForm.Image"/>CADImage    
		/// </summary>
#else
		/// <summary>
		/// Sets current properties for <see cref="MainForm.Image"/>CADImage for loading file
		/// </summary>
#endif
		#endregion
		public void SetCADImageOptions()
		{						
			cadImage.UseWinEllipse = false;					
			cadImage.IsShowLineWeight = true;						
			cadImage.DrawMode = CADDrawMode.Normal;
			this.cadImage.UseDoubleBuffering = false;			
			if(cadPictBox.BackColor == Color.White)
				White_Click();
			else 
				Black_Click();						
			this.Cursor = Cursors.Default;						
			SetLayList();	
			ObjEntity.cadImage = cadImage;						
			DoResize();			
			SetPictureBoxPosition(Position);	
			this.createCircleForm.Image = this.cadImage;
			this.createLineForm.Image = this.cadImage;
			this.createPolyForm.Image = this.cadImage;
			this.createTextForm.Image = this.cadImage;
			this.createMTextForm.Image = this.cadImage;
		}

		#region Help
		/// <summary>
		/// Displays a CAD image in black and white colors.
		/// </summary>
		#endregion Help
		public void DoBlackColor()
		{
			if(cadImage == null) 
				return;
			cadImage.DrawMode = CADDrawMode.Black;
			this.drawMode = false;
			cadPictBox.Invalidate();
		}

		private void SetPictureBoxPosition(PointF value)
		{
			int w1, h1;
			if(value.X > 0.0f)
				w1 = 0;
			else
				w1 = (int)Math.Abs(value.X);
			if(w1 > this.cadPictBox.VirtualSize.Width)
				w1 = this.cadPictBox.VirtualSize.Width;
			if(value.Y > 0.0f)
				h1 = 0;
			else
				h1 = (int)Math.Abs(value.Y);
			if(h1 > this.cadPictBox.VirtualSize.Height)
				h1 = this.cadPictBox.VirtualSize.Height;
			this.cadPictBox.SetPositionNoInvalidate(new Point(w1, h1));
		}
		
		private void NotSupportedInCurrentVersion()
		{
			this.Cursor = Cursors.Default;
			cadPictBox.Visible = true;
			stBar.Panels[0].Text = ApplicationConstants.notsupportedstr;
			MessageBox.Show(ApplicationConstants.notsupportedextstr, ApplicationConstants.appnamestr);
			return;
		}

		#region Help
		/// <summary>
		/// Enables or disables buttons of the toolbar and menu items of the main menu.
		/// </summary>
		/// <param name="aVal"><b>true</b> if to enable the buttons and menu items; <b>false</b> if to disable them.</param>
		/// <remarks>The buttons and menu items are enabled after loading a CAD file and become disabled after closing the file.</remarks>
		#endregion Help
		public void EnableButton(bool aVal)
		{
			for(int i = 1; i < tlbTool.Buttons.Count; i++)
				tlbTool.Buttons[i].Enabled = aVal;			
			#if Export
				this.miSaveAsDXF.Enabled = aVal;
			#endif
			
		}

		private void ZoomIn_Click(object sender, System.EventArgs e)
		{
			DoZoomIn();
		}

		#region Help
		/// <summary>
		/// Increases a scale of the CAD image in two times.
		/// </summary>
		#endregion Help
		public void DoZoomIn()
		{
			if(cadImage == null) 
				return;
			old_Pos = new PointF(cadPictBox.ClientRectangle.Width / 2.0f, cadPictBox.ClientRectangle.Height / 2.0f);
			sc = sc * 2.0f;
			cadPictBox.Invalidate();			
		}

		private void ZoomOut_Click(object sender, System.EventArgs e)
		{
			DoZoomOut();
		}

		#region Help
		/// <summary>
		/// Decreases a scale of the CAD image in two times.
		/// </summary>
		#endregion Help
		public void DoZoomOut()
		{
			if(cadImage == null) return;
			old_Pos = new PointF(cadPictBox.ClientRectangle.Width / 2.0f, cadPictBox.ClientRectangle.Height / 2.0f);
			sc = sc / 2.0f;
			cadPictBox.Invalidate();			
		}

		private void cadPictBox_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			if(this.cadImage == null)
				return;
			DrawCADImage(e.Graphics);
			stBar.Panels[1].Text = RealScale;							
		}

		#region Help
		/// <summary>
		/// Draws a CAD image from the loaded CAD file.
		/// </summary>
		/// <param name="gr">A <see cref="System.Drawing.Graphics">Graphics</see> object used to draw the CAD entities.</param>
		#endregion Help
		public void DrawCADImage(Graphics gr)
		{						
			try
			{								
				Shift();
				RectangleF tmp = ImageRectangleF;											
				SetSizePictureBox(new Size((int)tmp.Width, (int)tmp.Height));																								
				cadImage.Draw(gr, tmp);								
			}
			catch
			{
				return;
			}			
		}

		#region Help
#if RUSHELP
		/// <summary>
		///      <see cref="CADImport.FaceModule.CADPictureBox">CADPictureBox</see>
		/// </summary>
		/// <param name="sz"> </param>
#else
		/// <summary>
		/// Sets scroll region for <see cref="CADImport.FaceModule.CADPictureBox">CADPictureBox</see>
		/// </summary>
		/// <param name="sz">Size to set</param>
#endif
		#endregion
		public void SetSizePictureBox(Size sz)
		{		
			this.cadPictBox.SetVirtualSizeNoInvalidate(sz);			
		}

		private void cadPictBox_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			this.detRMouseDown = false;
			cadPictBox.Cursor = Cursors.Default;
			if(this.clipRectangle.Enabled)
				if((this.clipRectangle.Type == RectangleType.Zooming))
				{
					if((this.clipRectangle.ClientRectangle.Width <= 10)||
						(this.clipRectangle.ClientRectangle.Height <= 10))
					{ 
						cadPictBox.Invalidate(); 
						return;
					}
					if(this.sc > 1000) 
					{
						cadPictBox.Invalidate(); 
						return; 
					}
					float tmp1 = visibleArea.Width / this.clipRectangle.ClientRectangle.Width;
					float tmp2 = visibleArea.Height / this.clipRectangle.ClientRectangle.Height;
					if(tmp1 > tmp2) 
						tmp1 = tmp2;
					pos.X += this.clipRectangle.ClientRectangle.Width / 2.0f; 
					pos.Y += this.clipRectangle.ClientRectangle.Height / 2.0f; 
					this.Zoom(tmp1);
				}
			cadPictBox.Invalidate();
		}

		private void cadPictBox_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if((e.Button == MouseButtons.Right)||(e.Button == MouseButtons.Middle))
			{
				MainForm.ActiveForm.Cursor = Cursors.Hand;
				cX = e.X;
				cY = e.Y;
				detRMouseDown = true;
			}
			else
			{
				if(this.cadImage != null)
				{
					this.clipRectangle.EnableRect(RectangleType.Zooming, new Rectangle(e.X, e.Y, 0, 0));
				}
			}
		}

		private void cadPictBox_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			cadPictBox.Focus();
			if(cadImage == null) 
				return;
			if(this.detRMouseDown)
			{
				pos.X -= (cX - e.X);
				pos.Y -= (cY - e.Y);
				cX = e.X;
				cY = e.Y;
				cadPictBox.Invalidate();
				SetPictureBoxPosition(Position);
			}
			old_Pos = new PointF(e.X, e.Y);
			DPoint vPt = GetRealPoint(e.X, e.Y);
			stBar.Panels[2].Text = vPt.X.ToString() + ApplicationConstants.sepstr2 + vPt.Y.ToString() + 
								   ApplicationConstants.sepstr2 + vPt.Z.ToString();			
		}

		#region Help
		/// <summary>
		/// Gets a three-dimensional point of the current CAD image from the specified screen point.
		/// </summary>
		/// <param name="x">An X coordinate of the screen point.</param>
		/// <param name="y">A Y coordinate of the screen point.</param>
		/// <returns>A three-dimensional point.</returns>
		#endregion Help
		public DPoint GetRealPoint(int x, int y)
		{			
			RectangleF tmpRect = ImageRectangleF;	
			return CADConst.GetRealPoint(x, y, this.cadImage, tmpRect);
		}					

		#region Help
		/// <summary>
		/// Paints the CAD image background in the white color.
		/// </summary>
		#endregion Help
		public void White_Click()
		{
			tlbTool.Buttons[3].Pushed = true;
			tlbTool.Buttons[4].Pushed = false;
			cadPictBox.BackColor = Color.White;
			cadImage.DefaultColor = Color.Black;
			cadImage.BackgroundColor = Color.White;
			if(this.clipRectangle != null)
				this.clipRectangle.Color = Color.Black;
		}

		#region Help
		/// <summary>
		/// Paints the CAD image background in the black color.
		/// </summary>
		#endregion Help
		public void Black_Click()
		{
			tlbTool.Buttons[3].Pushed = false;
			tlbTool.Buttons[4].Pushed = true;
			cadPictBox.BackColor = Color.Black;
			cadImage.DefaultColor = Color.White;
			cadImage.BackgroundColor = Color.Black;
			if(this.clipRectangle != null)
				this.clipRectangle.Color = Color.White;
		}

		private void tlbTool_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			if((cadImage == null)&&(tlbTool.Buttons.IndexOf(e.Button) == 8)) 
				return;
			switch(tlbTool.Buttons.IndexOf(e.Button))
			{
				case 0:
					this.ShowLoadFileDialog();						
					break; 
				case 1:
					DoZoomIn();
					break; 
				case 2:
					DoZoomOut();
					break;
				case 3:
					White_Click();
					break;
				case 4:
					Black_Click();
					break;
				case 5:
					SetLayList();
					lForm.ShowDialog();
					break;
				case 6:	
					this.ShowAddEntityDialog(this.numlastadd);			
					break;
				case 7:
					SaveAsImage();
					break;
				case 8:
					if(MessageBox.Show(ApplicationConstants.msgstr, ApplicationConstants.appconst, MessageBoxButtons.YesNo) == DialogResult.Yes)
						ChangeTextsType(! this.useSHXFonts);
					break;
			}
			this.ChangeControlState();
		}

		private void ChangeTextsType(bool val)
		{
			useSHXFonts = val;
			CADConst.UseSHXFonts = useSHXFonts;
			CADConst.UseTTFFonts = ! useSHXFonts;
			CADConst.UseMultyTTFFonts = ! useSHXFonts;			
			this.ReOpen();
		}
	
		#region Help
		/// <summary>
		/// Reopens a file after selecting or deselecting SHX fonts.
		/// </summary>
		/// <remarks>This method is also called when the paths to the 
		/// files containing SHX fonts have been edited in the <b>SHX Paths</b> window 
		/// invoked by the <b>SHX Fonts</b> menu item.</remarks>
		#endregion Help
		public void ReOpen()
		{
			if(dlgOpenFile.FileName != null)				
				this.LoadFile(dlgOpenFile.FileName);
		}
		#region Help
		/// <summary>
		/// Sets the scale and position of the CAD image to its original values.
		/// </summary>
		#endregion Help
		public void ResetScaling()
		{
			sc = 1.0f;
			prev_sc = 1.0f;
			pos.X = (cadPictBox.ClientRectangle.Width - visibleArea.Width) / 2.0f;
			pos.Y = (cadPictBox.ClientRectangle.Height - visibleArea.Height) / 2.0f;
			cadPictBox.Invalidate();
		}

		private void AddRecordToLayerList(string name, Color color, bool visible, bool freeze)
		{
			ListItem lstItem = new ListItem();			
			lstItem.Text = name;								
			lForm.LayerList.Items.Add(lstItem);
			lstItem = new ListItem();
			if(visible)				
				lstItem.ImageIndex = 7;							
			else
				lstItem.ImageIndex = 8;							
			lForm.LayerList.Items.Add(lstItem);
			lstItem = new ListItem();
			if(freeze)
				lstItem.ImageIndex = 3;				
			else
				lstItem.ImageIndex = 4;				
			lForm.LayerList.Items.Add(lstItem);	
			lstItem = new ListItem();
			lstItem.DrawColorRect = true;
			lstItem.BackColor = color;			
			lForm.LayerList.Items.Add(lstItem);			
		}

		#region Help
		/// <summary>
		/// Sets <see cref="CADImport.CADLayer.Visible">CADLayer.Visible</see> parameter for the layer
		/// by the specified <see cref="CADImport.CADLayer">layer</see> index
		/// </summary>
		/// <param name="visible">Setting value</param>
		/// <param name="index">An index of the layer in the <see cref="CADImport.CADConverter.Layers"></see> collection</param>
		#endregion
		public void SetVisibleLayer(bool visible, int index)
		{
			this.cadImage.SetLayerVisible(index, visible, lForm.detInvalidate);						
			cadPictBox.Invalidate();			
		}
		#region Help
		/// <summary>
		/// Sets <see cref="CADImport.CADLayer.Frozen">CADLayer.Frozen</see> parameter for the layer
		/// by the specified <see cref="CADImport.CADLayer">layer</see> index
		/// </summary>
		/// <param name="visible">Setting value</param>
		/// <param name="index">An index of the layer in the <see cref="CADImport.CADConverter.Layers">layers</see> collection</param>
		#endregion
		public void SetFreezeLayer(bool visible, int index)
		{
			this.cadImage.SetLayerFreeze(index, visible, lForm.detInvalidate);						
			cadPictBox.Invalidate();			
		}

		#region Help
		/// <summary>
		/// Sets layer color by the specified <see cref="CADImport.CADLayer">layer</see> index
		/// </summary>
		/// <param name="color">Setting <see cref="System.Drawing.Color">color</see></param>
		/// <param name="index">An index of the layer in the <see cref="CADImport.CADConverter.Layers"></see> collection</param>
		#endregion
		public void SetColorLayer(Color color, int index)
		{
			this.cadImage.SetLayerColor(index, color, lForm.detInvalidate);						
			cadPictBox.Invalidate();
		}		
		
		private void ListSelect(ArrayList item, ListEventSelectArgs e)
		{
			ListItem itm1 =((ListItem)lForm.LayerList.Items[e.ItemIndex]);
			int index = e.ItemIndex / lForm.LayerList.Columns.Count;		
			switch(itm1.ImageIndex)
			{
				case 7:
					itm1.ImageIndex = 8;	
					SetVisibleLayer(false, index);				
					break;
				case 8:
					itm1.ImageIndex = 7;										
					SetVisibleLayer(true, index);
					break;
				case 4:
					itm1.ImageIndex = 3;
					SetFreezeLayer(true, index);
					break;
				case 3:
					itm1.ImageIndex = 4;
					SetFreezeLayer(false, index);
					break;
			}
			if(itm1.DrawColorRect)
			{
				this.setColorDlg.Color = itm1.BackColor;
				if(this.setColorDlg.ShowDialog() == DialogResult.OK)
				{
					SetColorLayer(this.setColorDlg.Color, index);	
					itm1.BackColor = this.setColorDlg.Color;
					this.lForm.LayerList.InvalidateCell(e.ItemIndex);			
				}
			}
			lForm.LayerList.InvalidateCell(e.ItemIndex);
		}

		private void SetLayList()
		{
			if((lForm.LayerList.Items.Count != 0)||(cadImage == null))
				return;
			ObjEntity.LayersList = new string[cadImage.LayersCount];			
			Color colorLay;					
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr1, ApplicationConstants.namestr, 0));
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr2, ApplicationConstants.visiblestr, 0));
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr4, ApplicationConstants.freezestr, 0));
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr3, ApplicationConstants.colorstr, 0));			
			string tmp;
			for(int I = 0; I < cadImage.LayersCount; I++)
			{
				tmp = cadImage.GetLayerName(I);
				colorLay = cadImage.GetLayerColor(I);
				ObjEntity.LayersList[I] = tmp;
				AddRecordToLayerList(tmp, colorLay, cadImage.GetLayerVisible(I), cadImage.GetLayerFreeze(I));				
			}		
			lForm.LayerList.SetAutoSize();
			lForm.LayerList.SelectItemEvent += listItemSelectDel;			
		}

		#region Help
		/// <summary>
		/// Resizes the CAD image to fit drawing's bounds.
		/// </summary>
		/// <remarks>This method is invoked when changing layout or working with 3D Orbit tool.</remarks>
		#endregion Help
		public void DoResize()
		{			
			if(cadImage == null) 
				return;
			float wh = (float)(cadImage.AbsWidth / cadImage.AbsHeight); 
			float new_wh = (float)cadPictBox.ClientRectangle.Width / (float)cadPictBox.ClientRectangle.Height;
			visibleArea = cadPictBox.ClientSize;
			if(new_wh > wh)
				visibleArea.Width	= (int)(visibleArea.Height * wh);
			else 
			{
				if(new_wh < wh)
					visibleArea.Height = (int)(visibleArea.Width / wh);
				else
					visibleArea = cadPictBox.ClientSize;
			}
			pos.X = (cadPictBox.ClientRectangle.Width - visibleArea.Width) / 2.0f;
			pos.Y = (cadPictBox.ClientRectangle.Height - visibleArea.Height) / 2.0f;				
			cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Closes a currently open CAD file.
		/// </summary>
		#endregion Help
		public void CloseFile()
		{
			cadImage.Dispose();
			cadImage = null;
			EnableButton(false);		
			this.cadPictBox.Invalidate();
		}
		
		#region Help
		/// <summary>
		/// Makes the layers of the loaded CAD image visible or invisible.
		/// </summary>
		/// <param name="sender">A <see cref="System.Windows.Forms.CheckedListBox">CheckedListBox</see> that initializes changing a visibility of the layers.</param>
		/// <param name="e">An <see cref="System.Windows.Forms.ItemCheckEventArgs">ItemCheckEventArgs</see> object that provides data for the event of changing a layer visibility.</param>
		#endregion Help
		public void chLay_ItemCheck(object sender, System.Windows.Forms.ItemCheckEventArgs e)
		{
			if(e.NewValue == CheckState.Checked)
				cadImage.SetLayerVisible(e.Index, true, lForm.detInvalidate);
			else if(e.NewValue == CheckState.Unchecked) 
				cadImage.SetLayerVisible(e.Index, false, lForm.detInvalidate);
			cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Invokes a <see cref="System.Windows.Forms.SaveFileDialog">SaveFileDialog</see> box where 
		/// a user can select a directory and format for saving the current CAD image.
		/// </summary>
		/// <remarks><p>This method can be invoked from the toolbar and from the main menu.</p>
		/// <p>Images can be saved in the following formats: JPG, BMP, TIFF, GIF, EMF.</p></remarks>
		#endregion Help
		public void SaveAsImage()
		{
			if(cadImage == null) 
				return;
			if(this.cadImage.FileInf != null)
				dlgSaveImg.FileName = this.cadImage.FileInf.Name.Replace(this.cadImage.FileInf.Extension, string.Empty);
			if(dlgSaveImg.ShowDialog() != DialogResult.OK) 
				return;
			DRect tmpRect = new DRect(0, 0, VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);
			ImageFormat imgFrm = ImageFormat.Bmp;
			if(dlgSaveImg.FileName != null) 
			{
				string tmp = dlgSaveImg.FileName;	
				if(tmp.ToUpper().IndexOf(ApplicationConstants.jpgextstr) != -1)
					imgFrm = ImageFormat.Jpeg;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.bmpextstr) != -1)
					imgFrm = ImageFormat.Bmp;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.tiffextstr) != -1)
					imgFrm = ImageFormat.Tiff;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.gifextstr) != -1)
					imgFrm = ImageFormat.Gif;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.emfextstr) != -1)
					imgFrm = ImageFormat.Emf;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.wmfextstr) != -1)
					imgFrm = ImageFormat.Wmf;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.pngextstr) != -1)
					imgFrm = ImageFormat.Png;				
				if(tmp.ToUpper().IndexOf(ApplicationConstants.dxfextstr) != -1)
				{
					SaveAsDXF(dlgSaveImg.FileName);
					return;
				}
			}			
			if(! this.clipRectangle.Enabled)
			{
				rasterSizeForm.Image = this.cadImage;
				rasterSizeForm.CurrentSize = tmpRect;
				rasterSizeForm.SaveFileName = dlgSaveImg.FileName;
				if(rasterSizeForm.ShowDialog() == DialogResult.OK)
					tmpRect = rasterSizeForm.SizeImage;
			}						
			int tmpNumberOfPartsInCircle = this.cadImage.NumberOfPartsInCircle;
			this.cadImage.NumberOfPartsInCircle = CADConst.SetNumberOfPartsInCurve(tmpRect);
			if(this.clipRectangle.Enabled)
			{				
				if((imgFrm == ImageFormat.Emf)||(imgFrm == ImageFormat.Wmf))				
					cadImage.ExportToMetafile(dlgSaveImg.FileName, tmpRect, this.clipRectangle.ClientRectangle);				
				else
					cadImage.SaveToFile(dlgSaveImg.FileName, imgFrm, tmpRect, this.clipRectangle.ClientRectangle, rasterSizeForm.ImagePixelFormat);				
			}
			else
			{				
				if((imgFrm == ImageFormat.Emf)||(imgFrm == ImageFormat.Wmf))
					cadImage.ExportToMetafile(dlgSaveImg.FileName, tmpRect);
				else
					cadImage.SaveToFile(dlgSaveImg.FileName, imgFrm, tmpRect, rasterSizeForm.ImagePixelFormat);				
			}						
			this.cadImage.NumberOfPartsInCircle = tmpNumberOfPartsInCircle;
		}	

		private void MainForm_Deactivate(object sender, System.EventArgs e)
		{
			this.detRMouseDown = false;
		}

		private void MainForm_Resize(object sender, System.EventArgs e)
		{
			DoResize();
		}

		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			#region protect
			#if protect
			#if floatprotect
			if(Protection.LicenseType == LicenseType.Floating)
			{
				this.floatLicFrm.DisableFloating();
			}
			else
			{
				Protection.CloseApplication();
			}
			#else
				Protection.CloseApplication();
			#endif
			#endif
			#endregion
		}

		private void MainForm_Load(object sender, System.EventArgs e)
		{			
			frmSHX.AddDefaultSHXPaths();
			visibleArea = cadPictBox.ClientSize;
			#region protect
#if protect
#if floatprotect
			if(Protection.LicenseType == LicenseType.Floating)
			{
				this.floatLicFrm.FloatingLicRegister();
			}
			else	
			{
				Protection.Register();
			}
#else
			Protection.Register();
#endif
#endif
			#endregion
			if(openFileName != null)							
				this.LoadFile(openFileName);												
			#region demo
			#if Demo
				DemoForm d1 = new DemoForm();
				d1.ShowDialog();
			#endif
			#endregion demo
			this.ChangeControlState();
		}

		private void ShowAddEntityDialog(int num)
		{
			DialogResult res = DialogResult.No;
			switch(num)
			{
				case 0:
					res = this.createCircleForm.ShowDialog();
					break;
				case 1:
					res = this.createLineForm.ShowDialog();
					break;
				case 2:
					res = this.createPolyForm.ShowDialog();
					break;
				case 3:
					res = this.createTextForm.ShowDialog();
					break;
				case 4:
					res = this.createMTextForm.ShowDialog();
					break;
			}
			if(res == DialogResult.OK)
				this.cadPictBox.Invalidate();
		}

		private void addEntity_Click(object sender, EventArgs e)
		{
			int val = (sender as MenuItem).Index;
			this.numlastadd = val;
			ShowAddEntityDialog(val);
		}				

		private void miAbout_Click(object sender, System.EventArgs e)
		{
			this.aboutFrm.ShowDialog();
		}

		private void miOpenFile_Click(object sender, System.EventArgs e)
		{
			this.ShowLoadFileDialog();	
			this.ChangeControlState();
		}

		private void ChangeControlState()
		{
			bool detNullImg = cadImage != null;
			EnableButton(detNullImg);
			bool tmp = false;
			if(cadPictBox.BackColor == Color.White)
				tmp = true;			
			if(detNullImg)
			{
				this.tlbWhite.Pushed = tmp;
				this.tlbBlack.Pushed = ! tmp;				
			}
			else 
			{
				this.tlbWhite.Pushed = false;
				this.tlbBlack.Pushed = false;				
			}						
			this.tlbShx.Pushed = this.useSHXFonts;															
		}

		private void miExit_Click(object sender, System.EventArgs e)
		{
			Close();
		}

		private void miSaveAsDXF_Click(object sender, System.EventArgs e)
		{
			if(this.cadImage == null) 
				return;
			if(this.dlgSaveDXF.ShowDialog() != DialogResult.OK) 
				return;
			SaveAsDXF(this.dlgSaveDXF.FileName);
		}

		private void SaveAsDXF(string fName)
		{
			#region Export
#if Export
			if(cadImage == null) 
				return;
			if(cadImage is CADRasterImage)
			{
				if(cadImage.FileInf != null)
					if(cadImage.FileInf.Extension.ToUpper() == ApplicationConstants.emfextstr)
						cadImage.EmfConvertToDXF(this.cadImage.FileInf.FullName, this.cadPictBox.Handle, fName);
				return;
			}
			CADtoDXF vExp = new CADtoDXF(cadImage);
			vExp.SaveToFile(fName);						
#else
				MessageBox.Show(ApplicationConstants.notsupportedextstr, ApplicationConstants.appnamestr);
#endif
			#endregion Export			
		}

		private void miSHXForm_Click(object sender, System.EventArgs e)
		{
			this.frmSHX.ShowDialog();
		}

		private void miRegister_Click(object sender, System.EventArgs e)
		{
			#region protect
			#if protect
				regFrm.ShowDialog();
			#endif
			#endregion protect
		}

		private void miFloatLic_Click(object sender, System.EventArgs e)
		{
			#region protect
			#if floatprotect
				floatLicFrm.ShowDialog();
			#endif
			#endregion
		}
	}

	internal abstract class ApplicationConstants
	{
		public static readonly string defaultstr;
		public static readonly string languagepath;
		public static readonly string loadfilestr;
		public static readonly string msgBoxDebugCaption;
		public static readonly string sepstr;
		public static readonly string notsupportedstr;
		public static readonly string notsupportedextstr;
		public static readonly string appnamestr;
		public static readonly string sepstr2;
		public static readonly string jpgextstr;
		public static readonly string bmpextstr;
		public static readonly string tiffextstr;
		public static readonly string gifextstr;
		public static readonly string emfextstr;
		public static readonly string wmfextstr;
		public static readonly string pngextstr;			
		public static readonly string dxfextstr;
		public static readonly string lngextstr;
		public static readonly string languagestr;
		public static readonly string languageIDstr;
		public static readonly string backcolorstr;
		public static readonly string blackstr;
		public static readonly string showentitystr;
		public static readonly string drawmodestr;
		public static readonly string truestr;
		public static readonly string shxpathcnstr;
		public static readonly string installstr;
		public static readonly string sepstr3;
		public static readonly string typelcstr;
		public static readonly string floatingstr;
		public static readonly string hoststr;
		public static readonly string portstr;
		public static readonly string appconst;
		public static readonly string blackstr2;
		public static readonly string whitestr;
		public static readonly string registerstr;
		public static readonly string headstr1;
		public static readonly string headstr2;
		public static readonly string headstr3;
		public static readonly string headstr4;
		public static readonly string namestr;
		public static readonly string visiblestr;
		public static readonly string freezestr;
		public static readonly string colorstr;			
		public static readonly string msgstr;	
		public static readonly string msgstr2;	
		public static readonly string onestr;
		public static readonly string onestr2;
		public static readonly string onestr3;
		public static readonly string zerostr;
		public static readonly string lstr;
		public static readonly string lstr2;
		public static readonly string font1str;
		public static readonly string sepstr4;
		public static readonly string boldstr;
		public static readonly string regularstr;
		public static readonly string underlinedstr;
		public static readonly string italicstr;
		public static readonly NumberFormatInfo numInfo;

		static ApplicationConstants()
		{
			defaultstr = "Default";
			languagepath = "LanguagePath";
			loadfilestr = "Loading file...";
			msgBoxDebugCaption = "Debug application message";
			msgstr = "If you proceed with this operation, all added objects will be lost. Continue?";
			msgstr2 = "Can't add a polyline with no vertices. Please, add a vertex first.";
			sepstr = " - ";
			notsupportedstr = "not supported";
			notsupportedextstr = "Not supported in current version!";
			appnamestr = "CADImportNet Demo";
			sepstr2 = " : ";			
			jpgextstr = ".JPG";
			bmpextstr = ".BMP";
			tiffextstr = ".TIFF";
			gifextstr = ".GIF";
			emfextstr = ".EMF";
			wmfextstr = ".WMF";
			pngextstr = ".PNG";			
			dxfextstr = ".DXF";
			lngextstr = ".lng";
			languagestr = "Language";
			languageIDstr = "LanguageID";
			backcolorstr = "BackgroundColor";
			blackstr = "BLACK";
			showentitystr = "ShowEntity";
			drawmodestr = "drawMode";
			truestr = "TRUE";
			shxpathcnstr = "SHXPathCount";
			installstr = "Install";
			sepstr3 = ";";
			typelcstr = "Type_license";
			floatingstr = "floating";
			hoststr = "Host";
			portstr = "Port";
			appconst = "Application";
			blackstr2 = "Black";
			whitestr = "White";
			registerstr = "register";
			headstr1 = "Head1";
			headstr2 = "Head2";
			headstr3 = "Head3";
			headstr4 = "Head4";
			namestr = "Name";
			visiblestr = "Visible";
			freezestr = "Freeze";
			colorstr = "Color";			
			onestr = "0.1";
			onestr2 = "1.0";
			onestr3 = "1";
			zerostr = "0";
			lstr = "l";
			lstr2 = "L";
			font1str = "SansSerif";
			sepstr4 = ".";			
			boldstr = "bold";
			regularstr = "regular";
			italicstr = ", italic";
			underlinedstr = ", underlined";
			numInfo = new NumberFormatInfo();
		}
	}

	#region Documentation
	#if Documentation
	#region Help
	/// <summary>
	/// <p>The <B>Add Entity</B> application demonstrates how CAD entities can be added to the opened CAD image. After opening a CAD file clicking on 
	/// the <B>Add</B> button of the toolbar invokes the <B>Select Entity</B> form in which a user can select a type of entity to add. 
	/// The possible types include circle, line, polyLine, text, multiline text. After selecting an entity type and clicking the <B>OK</B> button 
	/// the respective form will appear. In this form a user can set the main properties of the selected entity and by clicking the <B>OK</B> 
	/// button to add the entity to the current CAD image.</p>
	/// <p><b>Note:</b> To compile this project you need the extended version of the CAD Import .NET library that allows accessing CAD entities directly. 
	/// You can get this version of the library free of charge by emailing your request to <a href="mailto:info@cadsofttools.com?subject=CAD%20Import%20.NET">info@cadsofttools.com</a>.</p>
	/// </summary>
	#endregion Help
	public class NamespaceDoc
	{

	}
	#endif
	#endregion
}
