using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using CADImport;
using CADImport.FaceModule;
using CADImport.CADImportForms;

namespace CADImportNetDemos.AddEntityDemo
{
	#region Help
	/// <summary>
	/// Represents a form in which the properties of the line are set 
	/// before adding it to the current CAD image.
	/// </summary>
	#endregion Help
	public class AddLineForm : System.Windows.Forms.Form
	{		
		private CADLine line;
		private ColorDialog setColorDlg;
		private CADImport.CADImage cadImage;	
		
		#region Help
		/// <summary>
		/// Invokes on changing of <see cref="CADImportNetDemos.AddEntityDemo.AddPolyForm.Image"/>image drawing area
		/// </summary>
		#endregion
		public event CADImportEventHandler ResizeImageEvent;

		private System.Windows.Forms.Label line1;
		private System.Windows.Forms.Label line2;
		private System.Windows.Forms.Label line3;
		private System.Windows.Forms.Label line4;
		private System.Windows.Forms.Label line5;
		private System.Windows.Forms.Label vertLine1;
		private System.Windows.Forms.Label vertLine2;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Button btnPickColor;
		private System.Windows.Forms.TextBox tbColor;
		private System.Windows.Forms.Label lbColor;
		private System.Windows.Forms.TextBox tbZ;
		private System.Windows.Forms.TextBox tbY;
		private System.Windows.Forms.TextBox tbX;
		private System.Windows.Forms.Label lbY;
		private System.Windows.Forms.Label lbZ;
		private System.Windows.Forms.Label lbX;
		private System.Windows.Forms.TextBox tbZ2;
		private System.Windows.Forms.TextBox tbY2;
		private System.Windows.Forms.TextBox tbX2;
		private System.Windows.Forms.Label lbFirst;
		private System.Windows.Forms.Label lbSec;
		private System.Windows.Forms.ComboBox cbLineWeight;
		private System.Windows.Forms.Label lbLineWeight;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.Button btnOK;		
		private System.ComponentModel.Container components = null;		
		
		#region Help
		/// <summary>
		/// Raises the <see cref="CADImportNetDemos.AddEntityDemo.AddLineForm.ResizeImageEvent"/>ResizeImageEvent event. 
		/// </summary>
		#endregion
		public void OnResizeImage()
		{
			if (ResizeImageEvent != null)
				ResizeImageEvent();
		}

		#region Help
		/// <summary>
		/// Gets or sets <see cref="CADImport.CADImage">CADImage</see> object where new entities are added
		/// </summary>
		#endregion
		public CADImage Image
		{
			get
			{
				return this.cadImage;
			}
			set
			{
				this.cadImage = value;
			}
		}

		#region Help
		/// <summary>
		/// Initializes a new instance of the <see cref=" CADImportNetDemos.AddEntityDemo.AddLineForm">AddLineForm</see> class.
		/// </summary>
		#endregion Help
		public AddLineForm()
		{
			InitializeComponent();
			setColorDlg = new ColorDialog();
			this.cbLineWeight.SelectedItem = ApplicationConstants.onestr;
			this.tbColor.Enabled = false;
			this.StartPosition = FormStartPosition.CenterScreen;
		}
		
		#region Help
		/// <summary>
		/// Cleans up any resources being used.
		/// </summary>
		/// <param name="disposing">A value indicating if both managed and unmanaged resources have to be released (<b>true</b>) or only unmanaged (<b>false</b>). 
		///</param>
		#endregion Help
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnPickColor = new System.Windows.Forms.Button();
			this.tbColor = new System.Windows.Forms.TextBox();
			this.lbColor = new System.Windows.Forms.Label();
			this.tbZ = new System.Windows.Forms.TextBox();
			this.tbY = new System.Windows.Forms.TextBox();
			this.tbX = new System.Windows.Forms.TextBox();
			this.lbY = new System.Windows.Forms.Label();
			this.lbZ = new System.Windows.Forms.Label();
			this.lbX = new System.Windows.Forms.Label();
			this.tbZ2 = new System.Windows.Forms.TextBox();
			this.tbY2 = new System.Windows.Forms.TextBox();
			this.tbX2 = new System.Windows.Forms.TextBox();
			this.lbFirst = new System.Windows.Forms.Label();
			this.lbSec = new System.Windows.Forms.Label();
			this.line1 = new System.Windows.Forms.Label();
			this.line2 = new System.Windows.Forms.Label();
			this.line3 = new System.Windows.Forms.Label();
			this.line4 = new System.Windows.Forms.Label();
			this.line5 = new System.Windows.Forms.Label();
			this.vertLine1 = new System.Windows.Forms.Label();
			this.vertLine2 = new System.Windows.Forms.Label();
			this.cbLineWeight = new System.Windows.Forms.ComboBox();
			this.lbLineWeight = new System.Windows.Forms.Label();
			this.btnCancel = new System.Windows.Forms.Button();
			this.btnOK = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// btnPickColor
			// 
			this.btnPickColor.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnPickColor.Location = new System.Drawing.Point(204, 15);
			this.btnPickColor.Name = "btnPickColor";
			this.btnPickColor.Size = new System.Drawing.Size(75, 25);
			this.btnPickColor.TabIndex = 7;
			this.btnPickColor.Text = "Pick a color";
			this.btnPickColor.Click += new System.EventHandler(this.pickColorButton_Click);
			// 
			// tbColor
			// 
			this.tbColor.BackColor = System.Drawing.Color.Black;
			this.tbColor.Location = new System.Drawing.Point(120, 15);
			this.tbColor.Multiline = true;
			this.tbColor.Name = "tbColor";
			this.tbColor.Size = new System.Drawing.Size(75, 25);
			this.tbColor.TabIndex = 6;
			this.tbColor.Text = "";
			// 
			// lbColor
			// 
			this.lbColor.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.lbColor.Location = new System.Drawing.Point(16, 15);
			this.lbColor.Name = "lbColor";
			this.lbColor.Size = new System.Drawing.Size(56, 24);
			this.lbColor.TabIndex = 5;
			this.lbColor.Text = "Color :";
			// 
			// tbZ
			// 
			this.tbZ.Location = new System.Drawing.Point(122, 181);
			this.tbZ.Name = "tbZ";
			this.tbZ.Size = new System.Drawing.Size(56, 20);
			this.tbZ.TabIndex = 13;
			this.tbZ.Text = "0,0";
			// 
			// tbY
			// 
			this.tbY.Location = new System.Drawing.Point(122, 149);
			this.tbY.Name = "tbY";
			this.tbY.Size = new System.Drawing.Size(56, 20);
			this.tbY.TabIndex = 12;
			this.tbY.Text = "0,0";
			// 
			// tbX
			// 
			this.tbX.Location = new System.Drawing.Point(122, 117);
			this.tbX.Name = "tbX";
			this.tbX.Size = new System.Drawing.Size(56, 20);
			this.tbX.TabIndex = 11;
			this.tbX.Text = "0,0";
			// 
			// lbY
			// 
			this.lbY.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbY.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbY.Location = new System.Drawing.Point(10, 149);
			this.lbY.Name = "lbY";
			this.lbY.Size = new System.Drawing.Size(98, 20);
			this.lbY.TabIndex = 15;
			this.lbY.Text = "Y coordinate :";
			this.lbY.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// lbZ
			// 
			this.lbZ.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbZ.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbZ.Location = new System.Drawing.Point(10, 181);
			this.lbZ.Name = "lbZ";
			this.lbZ.Size = new System.Drawing.Size(98, 20);
			this.lbZ.TabIndex = 16;
			this.lbZ.Text = "Z coordinate :";
			this.lbZ.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// lbX
			// 
			this.lbX.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbX.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbX.Location = new System.Drawing.Point(10, 117);
			this.lbX.Name = "lbX";
			this.lbX.Size = new System.Drawing.Size(98, 20);
			this.lbX.TabIndex = 14;
			this.lbX.Text = "X coordinate :";
			this.lbX.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// tbZ2
			// 
			this.tbZ2.Location = new System.Drawing.Point(210, 181);
			this.tbZ2.Name = "tbZ2";
			this.tbZ2.Size = new System.Drawing.Size(56, 20);
			this.tbZ2.TabIndex = 19;
			this.tbZ2.Text = "0,0";
			// 
			// tbY2
			// 
			this.tbY2.Location = new System.Drawing.Point(210, 149);
			this.tbY2.Name = "tbY2";
			this.tbY2.Size = new System.Drawing.Size(56, 20);
			this.tbY2.TabIndex = 18;
			this.tbY2.Text = "1000,0";
			// 
			// tbX2
			// 
			this.tbX2.Location = new System.Drawing.Point(210, 117);
			this.tbX2.Name = "tbX2";
			this.tbX2.Size = new System.Drawing.Size(56, 20);
			this.tbX2.TabIndex = 17;
			this.tbX2.Text = "1000,0";
			// 
			// lbFirst
			// 
			this.lbFirst.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbFirst.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.lbFirst.Location = new System.Drawing.Point(114, 88);
			this.lbFirst.Name = "lbFirst";
			this.lbFirst.Size = new System.Drawing.Size(64, 23);
			this.lbFirst.TabIndex = 20;
			this.lbFirst.Text = "First point";
			// 
			// lbSec
			// 
			this.lbSec.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbSec.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.lbSec.Location = new System.Drawing.Point(194, 88);
			this.lbSec.Name = "lbSec";
			this.lbSec.Size = new System.Drawing.Size(88, 23);
			this.lbSec.TabIndex = 21;
			this.lbSec.Text = "Second point";
			// 
			// line1
			// 
			this.line1.BackColor = System.Drawing.Color.Black;
			this.line1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.line1.Location = new System.Drawing.Point(13, 85);
			this.line1.Name = "line1";
			this.line1.Size = new System.Drawing.Size(266, 1);
			this.line1.TabIndex = 22;
			// 
			// line2
			// 
			this.line2.BackColor = System.Drawing.Color.Black;
			this.line2.Location = new System.Drawing.Point(14, 109);
			this.line2.Name = "line2";
			this.line2.Size = new System.Drawing.Size(266, 1);
			this.line2.TabIndex = 23;
			// 
			// line3
			// 
			this.line3.BackColor = System.Drawing.Color.Black;
			this.line3.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.line3.Location = new System.Drawing.Point(14, 141);
			this.line3.Name = "line3";
			this.line3.Size = new System.Drawing.Size(266, 1);
			this.line3.TabIndex = 24;
			// 
			// line4
			// 
			this.line4.BackColor = System.Drawing.Color.Black;
			this.line4.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.line4.Location = new System.Drawing.Point(14, 173);
			this.line4.Name = "line4";
			this.line4.Size = new System.Drawing.Size(266, 1);
			this.line4.TabIndex = 25;
			// 
			// line5
			// 
			this.line5.BackColor = System.Drawing.Color.Black;
			this.line5.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.line5.Location = new System.Drawing.Point(14, 205);
			this.line5.Name = "line5";
			this.line5.Size = new System.Drawing.Size(266, 1);
			this.line5.TabIndex = 26;
			// 
			// vertLine1
			// 
			this.vertLine1.BackColor = System.Drawing.Color.Black;
			this.vertLine1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.vertLine1.Location = new System.Drawing.Point(106, 85);
			this.vertLine1.Name = "vertLine1";
			this.vertLine1.Size = new System.Drawing.Size(1, 120);
			this.vertLine1.TabIndex = 27;
			// 
			// vertLine2
			// 
			this.vertLine2.BackColor = System.Drawing.Color.Black;
			this.vertLine2.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.vertLine2.Location = new System.Drawing.Point(189, 85);
			this.vertLine2.Name = "vertLine2";
			this.vertLine2.Size = new System.Drawing.Size(1, 120);
			this.vertLine2.TabIndex = 28;
			// 
			// cbLineWeight
			// 
			this.cbLineWeight.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cbLineWeight.Items.AddRange(new object[] {
															  "0.1",
															  "0.2",
															  "0.3",
															  "0.4",
															  "0.5",
															  "0.6",
															  "0.7",
															  "0.8",
															  "0.9",
															  "1.0",
															  "1.1",
															  "1.2",
															  "1.3",
															  "1.4",
															  "1.5",
															  "1.6",
															  "1.7",
															  "1.8",
															  "1.9",
															  "2.0",
															  "2.1",
															  "2.2",
															  "2.3",
															  "2.4",
															  "2.5",
															  "2.6",
															  "2.7",
															  "2.8",
															  "2.9",
															  "3.0",
															  "3.1",
															  "3.2",
															  "3.3",
															  "3.4",
															  "3.5",
															  "3.6",
															  "3.7",
															  "3.8",
															  "3.9",
															  "4.0",
															  "4.1",
															  "4.2",
															  "4.3",
															  "4.4",
															  "4.5",
															  "4.6",
															  "4.7",
															  "4.8",
															  "4.9",
															  "5.0",
															  "5.1",
															  "5.2",
															  "5.3",
															  "5.4",
															  "5.5",
															  "5.6",
															  "5.7",
															  "5.8",
															  "5.9",
															  "6.0",
															  "6.1",
															  "6.2",
															  "6.3",
															  "6.4",
															  "6.5",
															  "6.6",
															  "6.7",
															  "6.8",
															  "6.9",
															  "7.0",
															  "7.1",
															  "7.2",
															  "7.3",
															  "7.4",
															  "7.5",
															  "7.6",
															  "7.7",
															  "7.8",
															  "7.9",
															  "8.0",
															  "8.1",
															  "8.2",
															  "8.3",
															  "8.4",
															  "8.5",
															  "8.6",
															  "8.7",
															  "8.8",
															  "8.9",
															  "9.0",
															  "9.1",
															  "9.2",
															  "9.3",
															  "9.4",
															  "9.5",
															  "9.6",
															  "9.7",
															  "9.8",
															  "9.9",
															  "10.0"});
			this.cbLineWeight.Location = new System.Drawing.Point(120, 49);
			this.cbLineWeight.Name = "cbLineWeight";
			this.cbLineWeight.Size = new System.Drawing.Size(75, 21);
			this.cbLineWeight.TabIndex = 30;
			// 
			// lbLineWeight
			// 
			this.lbLineWeight.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbLineWeight.Location = new System.Drawing.Point(16, 53);
			this.lbLineWeight.Name = "lbLineWeight";
			this.lbLineWeight.Size = new System.Drawing.Size(80, 20);
			this.lbLineWeight.TabIndex = 29;
			this.lbLineWeight.Text = "Line weight :";
			// 
			// btnCancel
			// 
			this.btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnCancel.Location = new System.Drawing.Point(204, 221);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.Size = new System.Drawing.Size(75, 25);
			this.btnCancel.TabIndex = 32;
			this.btnCancel.Text = "Cancel";
			this.btnCancel.Click += new System.EventHandler(this.addLineCancel_Click);
			// 
			// btnOK
			// 
			this.btnOK.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnOK.Location = new System.Drawing.Point(120, 221);
			this.btnOK.Name = "btnOK";
			this.btnOK.Size = new System.Drawing.Size(75, 25);
			this.btnOK.TabIndex = 31;
			this.btnOK.Text = "OK";
			this.btnOK.Click += new System.EventHandler(this.addLineOK_Click);
			// 
			// label1
			// 
			this.label1.BackColor = System.Drawing.Color.Black;
			this.label1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label1.Location = new System.Drawing.Point(280, 85);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(1, 120);
			this.label1.TabIndex = 33;
			// 
			// label2
			// 
			this.label2.BackColor = System.Drawing.Color.Black;
			this.label2.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label2.Location = new System.Drawing.Point(13, 86);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(1, 120);
			this.label2.TabIndex = 34;
			// 
			// AddLineForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 253);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnOK);
			this.Controls.Add(this.cbLineWeight);
			this.Controls.Add(this.lbLineWeight);
			this.Controls.Add(this.vertLine2);
			this.Controls.Add(this.vertLine1);
			this.Controls.Add(this.line5);
			this.Controls.Add(this.line4);
			this.Controls.Add(this.line3);
			this.Controls.Add(this.line2);
			this.Controls.Add(this.line1);
			this.Controls.Add(this.lbSec);
			this.Controls.Add(this.lbFirst);
			this.Controls.Add(this.tbZ2);
			this.Controls.Add(this.tbY2);
			this.Controls.Add(this.tbX2);
			this.Controls.Add(this.tbZ);
			this.Controls.Add(this.tbY);
			this.Controls.Add(this.tbX);
			this.Controls.Add(this.tbColor);
			this.Controls.Add(this.btnPickColor);
			this.Controls.Add(this.lbColor);
			this.Controls.Add(this.lbY);
			this.Controls.Add(this.lbZ);
			this.Controls.Add(this.lbX);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.Name = "AddLineForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Add Line";
			this.ResumeLayout(false);

		}
		#endregion

		private void pickColorButton_Click(object sender, System.EventArgs e)
		{
			if(setColorDlg.ShowDialog() == DialogResult.OK)
			{
				this.tbColor.BackColor = setColorDlg.Color;
			}
		}

		private void addLineOK_Click(object sender, System.EventArgs e)
		{
			line = new CADLine();
			if(setColorDlg.Color == Color.Black)
				line.Color = CADConst.clNone;
			else
				line.Color = setColorDlg.Color;
			try
			{
				line.Point = new DPoint(Convert.ToDouble(this.tbX.Text), Convert.ToDouble(this.tbY.Text), Convert.ToDouble(this.tbZ.Text));
				line.Point1 = new DPoint(Convert.ToDouble(this.tbX2.Text), Convert.ToDouble(this.tbY2.Text), Convert.ToDouble(this.tbZ2.Text));
				line.LineWeight = Convert.ToSingle(this.cbLineWeight.Text, ApplicationConstants.numInfo);
				cadImage.Converter.GetSection(ConvSection.Entities).AddEntity(line);
				cadImage.Converter.OnCreate(line);
				cadImage.Converter.Loads(line);			
				this.OnResizeImage();
			}
			catch(FormatException exception)
			{
				MessageBox.Show(exception.Message, "Add Entity Demo", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
			}
			this.DialogResult = DialogResult.OK;
			this.Close();
		}

		private void addLineCancel_Click(object sender, System.EventArgs e)
		{
			this.DialogResult = DialogResult.Cancel;
			this.Close();
		}
	}
}
