Imports System
Imports System.Drawing
Imports System.Xml
Imports System.Xml.XPath
Imports CADImport

Public Class XMLImporter
    ' Methods
    Private Sub AddRealPointNode(ByVal val As DPoint)
        Dim pt As DPoint = Me.cadParams.matrix.PtXMat(val)
        Me.attr = Me.doc.CreateAttribute("X")
        Me.attr.Value = pt.X.ToString
        Me.nodeParam.Attributes.Append(Me.attr)
        Me.attr = Me.doc.CreateAttribute("Y")
        Me.attr.Value = pt.Y.ToString
        Me.nodeParam.Attributes.Append(Me.attr)
        Me.attr = Me.doc.CreateAttribute("Z")
        Me.attr.Value = pt.Z.ToString
        Me.nodeParam.Attributes.Append(Me.attr)
    End Sub

    Private Sub AddScreenPointNode(ByVal val As Point)
        Me.attr = Me.doc.CreateAttribute("X")
        Me.attr.Value = val.X.ToString
        Me.nodeParam.Attributes.Append(Me.attr)
        Me.attr = Me.doc.CreateAttribute("Y")
        Me.attr.Value = val.Y.ToString
        Me.nodeParam.Attributes.Append(Me.attr)
    End Sub

    Private Sub AddValueNode(ByVal val As String)
        Me.attr = Me.doc.CreateAttribute("Value")
        Me.attr.Value = val
        Me.nodeParam.Attributes.Append(Me.attr)
    End Sub

    Private Sub CreateXmlDocument()
        If (Not Me.doc Is Nothing) Then
            Me.doc.RemoveAll()
            Me.doc = Nothing
        End If
        Me.doc = New XmlDocument
    End Sub

    Public Sub CreateXMLRecords(ByVal fileName As String, ByVal sourceFile As String)
        If (Not Me.image Is Nothing) Then
            Me.CreateXmlDocument()
            Me.cadParams = New CADIterate
            Me.cadParams.matrix = New CADMatrix
            Me.cadParams.matrix.data(0, 0) = 1
            Me.cadParams.matrix.data(1, 1) = 1
            Me.cadParams.matrix.data(2, 2) = 1
            Me.image.Converter.AutoInsert = True
            Me.CreateXmlDocument()
            Me.baseNode = Me.doc.CreateElement("File")
            Me.attr = Me.doc.CreateAttribute("FileName")
            Me.attr.Value = sourceFile
            Me.baseNode.Attributes.Append(Me.attr)
            Me.nodeEntities = Me.doc.CreateElement("Entities")
            Me.baseNode.AppendChild(Me.nodeEntities)
            Me.image.Converter.Iterate(New CADEntityProc(AddressOf Me.ReadCADEntitiesParamsToXML), Me.cadParams)
            Me.doc.AppendChild(Me.baseNode)
            Me.doc.Save(fileName)
            Me.doc.RemoveAll()
        End If
    End Sub

    Private Sub ImportArcXML(ByVal arc As CADArc)
        Me.nodeParam = Me.doc.CreateElement("Point")
        Me.AddRealPointNode(arc.Point)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("StartAngle")
        Me.AddValueNode(arc.StartAngle.ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("EndAngle")
        Me.AddValueNode(arc.EndAngle.ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Radius")
        Me.AddValueNode(arc.Radius.ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
    End Sub

    Private Sub ImportAttdefXML(ByVal attdef As CADAttdef)
        Me.nodeParam = Me.doc.CreateElement("StartPoint")
        Me.AddRealPointNode(attdef.StartPoint)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Text")
        Me.AddValueNode(attdef.Text)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Angle")
        Me.AddValueNode(attdef.Rotation.ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
    End Sub

    Private Sub ImportCircleXML(ByVal circle As CADCircle)
        Me.nodeParam = Me.doc.CreateElement("Point")
        Me.AddRealPointNode(circle.Point)
        Me.nodeEntity.AppendChild(Me.nodeParam)
    End Sub

    Private Sub ImportEllipseXML(ByVal ellipse As CADEllipse)
        Me.nodeParam = Me.doc.CreateElement("Point")
        Me.AddRealPointNode(ellipse.Point)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("StartAngle")
        Me.AddValueNode(ellipse.StartAngle.ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("EndAngle")
        Me.AddValueNode(ellipse.EndAngle.ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Rx")
        Me.AddValueNode(ellipse.Radius.ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Ry")
        Me.AddValueNode((ellipse.Radius * ellipse.Ratio).ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
    End Sub

    Private Sub ImportHatchXML(ByVal hatch As CADHatch)
        Me.nodeParam = Me.doc.CreateElement("HatchName")
        Me.AddValueNode(hatch.HatchName)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Dim style As CADHatchStyle = hatch.HatchStyle
        Me.nodeParam = Me.doc.CreateElement("HatchStyle")
        Me.AddValueNode(ApplicationConstants.hatchStyle(CInt(style)))
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Elevation")
        Me.AddRealPointNode(hatch.Elevation)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Extrusion")
        Me.AddRealPointNode(hatch.Extrusion)
        Me.nodeEntity.AppendChild(Me.nodeParam)
    End Sub

    Private Sub ImportLineXML(ByVal line As CADLine)
        Me.nodeParam = Me.doc.CreateElement("StartPoint")
        Me.AddRealPointNode(line.Point)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("EndPoint")
        Me.AddRealPointNode(line.Point1)
        Me.nodeEntity.AppendChild(Me.nodeParam)
    End Sub

    Private Sub ImportPolyLineXML(ByVal polyline As CADPolyLine)
        Dim i As Integer
        For i = 0 To polyline.Count - 1
            Dim vertexNode As XmlNode = Me.doc.CreateElement(String.Format("Vertex{0}", i))
            Dim vertex As CADVertex = DirectCast(polyline.Entities.Item(i), CADVertex)
            Me.nodeParam = Me.doc.CreateElement("Bulge")
            Me.AddValueNode(vertex.Bulge.ToString)
            vertexNode.AppendChild(Me.nodeParam)
            Me.nodeParam = Me.doc.CreateElement("Point")
            Me.AddRealPointNode(vertex.Point)
            vertexNode.AppendChild(Me.nodeParam)
            Me.nodeEntity.AppendChild(vertexNode)
        Next i
    End Sub

    Private Sub ImportSolidXML(ByVal solid As CADSolid)
        Me.nodeParam = Me.doc.CreateElement("Point")
        Me.AddRealPointNode(solid.Point)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Point1")
        Me.AddRealPointNode(solid.Point1)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Point2")
        Me.AddRealPointNode(solid.Point2)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Point3")
        Me.AddRealPointNode(solid.Point3)
        Me.nodeEntity.AppendChild(Me.nodeParam)
    End Sub

    Private Sub ImportSplineXML(ByVal spline As CADSpline)
        Me.nodeParam = Me.doc.CreateElement("FitPointsCount")
        Me.AddValueNode(spline.FitCount.ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Dim cn As Integer = spline.FitCount
        Dim i As Integer
        For i = 0 To cn - 1
            Me.nodeParam = Me.doc.CreateElement(String.Format("FitPoint{0}", i))
            Me.AddRealPointNode(DirectCast(spline.Fit.Item(i), DPoint))
            Me.nodeEntity.AppendChild(Me.nodeParam)
        Next i
        cn = spline.ControlCount        
        For i = 0 To cn - 1
            Me.nodeParam = Me.doc.CreateElement(String.Format("ControlPoint{0}", i))
            Me.AddRealPointNode(spline.Controls.Item(i))
            Me.nodeEntity.AppendChild(Me.nodeParam)
        Next i
    End Sub

    Private Sub ImportTextXML(ByVal [text] As CADText)
        CADConst.DoScale2D((Me.cadParams))
        Me.nodeParam = Me.doc.CreateElement("StartPoint")
        Me.AddRealPointNode([text].StartPoint)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Text")
        Me.AddValueNode([text].Text)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Me.nodeParam = Me.doc.CreateElement("Angle")
        Me.AddValueNode([text].Rotation.ToString)
        Me.nodeEntity.AppendChild(Me.nodeParam)
    End Sub

    Private Sub ReadCADEntitiesParamsToXML(ByVal entity As CADEntity)
        Dim str As String = CADImport.FaceModule.CADImportFace.SetName(entity)
        Me.nodeEntity = Me.doc.CreateElement("Entity")
        Me.attr = Me.doc.CreateAttribute("ClassName")
        Me.attr.Value = String.Format("CADImport.CAD{0}", str)
        Me.nodeEntity.Attributes.Append(Me.attr)
        Me.attr = Me.doc.CreateAttribute("EntityName")
        Me.attr.Value = str
        Me.nodeEntity.Attributes.Append(Me.attr)
        Dim cl As Color = CADConst.EntColor(entity, Me.cadParams.insert)
        Me.attr = Me.doc.CreateAttribute("Color")
        Me.attr.Value = ApplicationConstants.GetColor(cl)
        Me.nodeEntity.Attributes.Append(Me.attr)
        Me.attr = Me.doc.CreateAttribute("LineWeight")
        Dim val As Double = CADConst.EntLineWeight(entity, Me.cadParams.insert)
        If (val = 0) Then
            val = Me.image.NullWidth
        End If
        Me.attr.Value = val.ToString
        Me.nodeEntity.Attributes.Append(Me.attr)
        CADConst.DoScale2D((Me.cadParams))
        Dim layer As CADLayer = CADConst.EntLayer(entity, Me.cadParams.insert)
        Me.nodeParam = Me.doc.CreateElement("Layer")
        str = "0"
        cl = CADConst.clNone
        If (Not layer Is Nothing) Then
            str = layer.Name
            cl = layer.Color
        End If
        Me.attr = Me.doc.CreateAttribute("Name")
        Me.attr.Value = str
        Me.nodeParam.Attributes.Append(Me.attr)
        Me.attr = Me.doc.CreateAttribute("Color")
        Me.attr.Value = ApplicationConstants.GetColor(cl)
        Me.nodeParam.Attributes.Append(Me.attr)
        Me.nodeEntity.AppendChild(Me.nodeParam)
        Select Case entity.EntType
            Case EntityType.Line, EntityType.Point
                Me.ImportLineXML(CType(entity, CADLine))
                Exit Select
            Case EntityType.Solid
                Me.ImportSolidXML(CType(entity, CADSolid))
                Exit Select
            Case EntityType.Circle
                Me.ImportCircleXML(CType(entity, CADCircle))
                Exit Select
            Case EntityType.Arc
                Me.ImportArcXML(CType(entity, CADArc))
                Exit Select
            Case EntityType.Ellipse
                Me.ImportEllipseXML(CType(entity, CADEllipse))
                Exit Select
            Case EntityType.Polyline, EntityType.LWPolyline
                Me.ImportPolyLineXML(CType(entity, CADPolyLine))
                Exit Select
            Case EntityType.Spline, EntityType.Leader
                Me.ImportSplineXML(CType(entity, CADSpline))
                Exit Select
            Case EntityType.Text
                Me.ImportTextXML(CType(entity, CADText))
                Exit Select
            Case EntityType.Attdef, EntityType.Attrib
                Me.ImportAttdefXML(CType(entity, CADAttdef))
                Exit Select
            Case EntityType.Hatch
                Me.ImportHatchXML(CType(entity, CADHatch))
                Exit Select
        End Select
        Me.nodeEntities.AppendChild(Me.nodeEntity)
    End Sub


    ' Properties
    Public Property Image() As CADImage
        Get
            Return Me.cadImg
        End Get
        Set(ByVal value As CADImage)
            Me.cadImg = value
        End Set
    End Property


    ' Fields
    Private attr As XmlAttribute
    Private baseNode As XmlNode
    Private cadParams As CADIterate
    Private doc As XmlDocument
    Private cadImg As CADImage
    Private nodeEntities As XmlNode
    Private nodeEntity As XmlNode
    Private nodeParam As XmlNode
End Class